@@ -7,24 +7,24 @@ from __future__ import (
 
 import numpy as np
 
-import requests
-
 from .query import DALResults, DALQuery, DALService, Record
 from .exceptions import DALServiceError
 from .vosi import AvailabilityMixin, CapabilityMixin
+from .params import find_param_by_keyword, get_converter
 
 from astropy.io.votable.tree import Param
 from astropy.units import Quantity, Unit
 from astropy.units import spectral as spectral_equivalencies
 
+from astropy.io.votable.tree import Resource, Group
+from astropy.utils.collections import HomogeneousList
+
 from ..utils.decorators import stream_decode_content
+from ..utils.http import session as s
 
 
 # monkeypatch astropy with group support in RESOURCE
 def _monkeypath_astropy_resource_groups():
-    from astropy.io.votable.tree import Resource, Group
-    from astropy.utils.collections import HomogeneousList
-
     old_group_unknown_tag = Group._add_unknown_tag
 
     def new_group_unknown_tag(self, iterator, tag, data, config, pos):
@@ -56,11 +56,12 @@ def _monkeypath_astropy_resource_groups():
         if tag == "GROUP":
             return self._add_group(iterator, tag, data, config, pos)
         else:
-            old_resource_unknown_tag(iterator, tag, data, config, pos)
+            old_resource_unknown_tag(self, iterator, tag, data, config, pos)
     Resource._add_unknown_tag = new_resource_unknown_tag
 
 
-_monkeypath_astropy_resource_groups()
+if not hasattr(Resource, 'groups'):
+    _monkeypath_astropy_resource_groups()
 
 __all__ = [
     "AdhocServiceResultsMixin", "DatalinkResultsMixin", "DatalinkRecordMixin",
@@ -68,6 +69,25 @@ __all__ = [
     "SodaRecordMixin", "SodaQuery"]
 
 
+def _get_input_params_from_resource(resource):
+    # get the group with name inputParams
+    group_input_params = next(
+        group for group in resource.groups if group.name == "inputParams")
+    # get only Param elements from the group
+    return {
+        _.name: _ for _ in group_input_params.entries if isinstance(_, Param)}
+
+
+def _get_params_from_resource(resource):
+    return {_.name: _ for _ in resource.params}
+
+
+def _get_accessurl_from_params(params):
+    if "accessURL" not in params:
+        raise DALServiceError("Datalink has no accessURL")
+    return params["accessURL"].value
+
+
 class AdhocServiceResultsMixin(object):
     """
     Mixing for adhoc:service functionallity for results classes.
@@ -109,6 +129,26 @@ class AdhocServiceResultsMixin(object):
         raise DALServiceError(
             "No Adhoc Service with ivo-id {}!".format(ivo_id))
 
+    def get_adhocservice_by_id(self, id_):
+        """
+        Return the adhoc service starting with the given service_def id.
+
+        Parameters
+        ----------
+        id_ : str
+           the service_def id of the service we want to have.
+
+        Returns
+        -------
+        Resource
+            The resource element describing the service.
+        """
+        for adhocservice in self.iter_adhocservices():
+            if adhocservice.ID == id_:
+                return adhocservice
+        raise DALServiceError(
+            "No Adhoc Service with service_def id {}!".format(id_))
+
 
 class DatalinkResultsMixin(AdhocServiceResultsMixin):
     """
@@ -139,7 +179,7 @@ class DatalinkRecordMixin(object):
     def getdataset(self, timeout=None):
         try:
             url = next(self.getdatalink().bysemantics('#this')).access_url
-            r = requests.get(url, stream=True, timeout=timeout)
+            r = s.get(url, stream=True, timeout=timeout)
             r.raise_for_status()
             return r.raw
         except (DALServiceError, StopIteration):
@@ -243,35 +283,35 @@ class DatalinkQuery(DALQuery):
                 ref="srcGroup"/>
         </GROUP>
         """
-        # get the group with name inputParams
-        group_input_params = next(
-            group for group in resource.groups if group.name == "inputParams")
+        input_params = _get_input_params_from_resource(resource)
         # get params outside of any group
-        dl_params = {_.name: _ for _ in resource.params}
-        # get only Param elements from the group
-        input_params = (
-            _ for _ in group_input_params.entries if isinstance(_, Param))
-
-        if "accessURL" not in dl_params:
-            raise DALServiceError("Datalink has no accessURL")
+        dl_params = _get_params_from_resource(resource)
+        accessurl = _get_accessurl_from_params(dl_params)
 
         query_params = dict()
-        for input_param in input_params:
+        for name, input_param in input_params.items():
             if input_param.ref:
-                query_params[input_param.name] = row[input_param.ref]
+                query_params[name] = row[input_param.ref]
             elif np.isscalar(input_param.value) and input_param.value:
-                query_params[input_param.name] = input_param.value
+                query_params[name] = input_param.value
             elif (
                     not np.isscalar(input_param.value) and
                     input_param.value.all() and
                     len(input_param.value)
             ):
-                query_params[input_param.name] = " ".join(
+                query_params[name] = " ".join(
                     str(_) for _ in input_param.value)
 
-        query_params.update(kwargs)
+        for name, query_param in kwargs.items():
+            try:
+                input_param = find_param_by_keyword(name, input_params)
+                converter = get_converter(input_param)
+                query_params[input_param.name] = converter.serialize(
+                    query_param)
+            except KeyError:
+                query_params[name] = query_param
 
-        return cls(dl_params["accessURL"].value, **query_params)
+        return cls(accessurl, **query_params)
 
     def __init__(
             self, baseurl, id=None, responseformat=None, **keywords):
@@ -412,6 +452,22 @@ class DatalinkResults(DatalinkResultsMixin, DALResults):
         except StopIteration:
             raise ValueError("No row with semantics #this found!")
 
+    def iter_procs(self):
+        """
+        iterate over all rows with a processing service
+        """
+        for row in self:
+            if row.service_def:
+                yield row
+
+    def get_first_proc(self):
+        """
+        returns the first datalink row with a processing service.
+        """
+        for proc in self.iter_procs():
+            return proc
+        raise IndexError("No processing service found in datalink result")
+
 
 class SodaRecordMixin(object):
     """
@@ -509,9 +565,24 @@ class DatalinkRecord(DatalinkRecordMixin, SodaRecordMixin, Record):
     @property
     def access_url(self):
         """
-        Link to data
+        Link to data or processing service
         """
-        return self.get("access_url", decode=True)
+        row_url = self.get("access_url", decode=True)
+
+        if not row_url:
+            proc_resource = self._results.get_adhocservice_by_id(
+                self.service_def)
+            dl_params = _get_params_from_resource(proc_resource)
+            row_url = _get_accessurl_from_params(dl_params)
+
+        return row_url
+
+    @property
+    def service_def(self):
+        """
+        reference to the service descriptor resource
+        """
+        return self.get("service_def", decode=True)
 
     @property
     def error_message(self):
@@ -559,6 +630,32 @@ class DatalinkRecord(DatalinkRecordMixin, SodaRecordMixin, Record):
 
         return self.access_url
 
+    def process(self, **kwargs):
+        """
+        calls the processing service and returns it's result as a file-like
+        object
+        """
+        proc_resource = self._results.get_adhocservice_by_id(self.service_def)
+        proc_query = DatalinkQuery.from_resource(self, proc_resource, **kwargs)
+        proc_stream = proc_query.execute_stream()
+        return proc_stream
+
+    @property
+    def params(self):
+        """
+        the access parameters of the service behind this datalink row.
+        """
+        proc_resource = self._results.get_adhocservice_by_id(self.service_def)
+        return proc_resource.params
+
+    @property
+    def input_params(self):
+        """
+        a list of input parameters for the service behind this datalink row.
+        """
+        proc_resource = self._results.get_adhocservice_by_id(self.service_def)
+        return list(_get_input_params_from_resource(proc_resource).values())
+
 
 class SodaQuery(DatalinkQuery):
     """
