@@ -45,8 +45,11 @@ from astropy.utils.exceptions import AstropyDeprecationWarning
 
 from .exceptions import (DALFormatError, DALServiceError, DALQueryError)
 
+from .. import samp
+
 from ..utils.decorators import stream_decode_content
-from ..utils.formatting import para_format_desc
+from ..utils.http import session as s
+
 
 if six.PY3:
     _mimetype_re = re.compile(
@@ -123,32 +126,8 @@ class DALService(object):
         q = DALQuery(self.baseurl, **keywords)
         return q
 
-    def describe(self, width=None):
-        """
-        Print a summary description of this service.
-
-        This includes the interface capabilities, and the content description
-        if it doesn't contains multiple data collections (in other words, it is
-        not a TAP service).
-        """
-        if len(self.tables) == 1:
-            description = next(self.tables.values()).description
-
-            if width:
-                description = para_format_desc(description, width)
-
-            print(description)
-            print()
-
-        capabilities = filter(
-            lambda x: not six.text_type(x.standardid).startswith(
-                'ivo://ivoa.net/std/VOSI'),
-            self.capabilities
-        )
-
-        for cap in capabilities:
-            cap.describe()
-            print()
+    def describe(self):
+        print('DAL Service at {}'.format(self.baseurl))
 
 
 class DALQuery(dict):
@@ -237,7 +216,7 @@ class DALQuery(dict):
         url = self.queryurl
         params = {k: v for k, v in self.items()}
 
-        r = requests.get(url, params=params, stream=True)
+        r = s.get(url, params=params, stream=True)
         return r
 
     def execute_votable(self):
@@ -297,7 +276,7 @@ class DALResults(object):
     @classmethod
     @stream_decode_content
     def _from_result_url(cls, result_url):
-        return requests.get(result_url, stream=True).raw
+        return s.get(result_url, stream=True).raw
 
     @classmethod
     def from_result_url(cls, result_url):
@@ -624,6 +603,15 @@ class DALResults(object):
             yield out
             pos += 1
 
+    def broadcast_samp(self, client_name=None):
+        """
+        Broadcast the table to ``client_name`` via SAMP
+        """
+        with samp.connection() as conn:
+            samp.send_table_to(
+                conn, self.to_table(),
+                client_name=client_name, name=self.queryurl)
+
     def cursor(self):
         """
         return a cursor that is compliant with the Python Database API's
@@ -761,9 +749,9 @@ class Record(Mapping):
             raise KeyError("no dataset access URL recognized in record")
 
         if timeout:
-            r = requests.get(url, stream=True, timeout=timeout)
+            r = s.get(url, stream=True, timeout=timeout)
         else:
-            r = requests.get(url, stream=True)
+            r = s.get(url, stream=True)
 
         r.raise_for_status()
         return r.raw
