@@ -33,6 +33,8 @@ from __future__ import (
 
 import re
 
+from pyvo.io.vosi.vodataservice import TableParam
+
 from astropy.extern import six
 from astropy.coordinates import SkyCoord
 from astropy.time import Time
@@ -41,6 +43,8 @@ from astropy.units import Quantity, Unit
 from .query import DALResults, DALQuery, DALService, Record, mime2extension
 from .adhoc import DatalinkResultsMixin, DatalinkRecordMixin, SodaRecordMixin
 
+from .. import samp
+
 __all__ = ["search", "SIAService", "SIAQuery", "SIAResults", "SIARecord"]
 
 
@@ -168,6 +172,20 @@ class SIAService(DALService):
         except AttributeError:
             return None
 
+    @property
+    def columns(self):
+        """
+        the available columns on this service
+        """
+        self._get_metadata()
+        fields = getattr(self, '_metadata', None).get_first_table().fields
+
+        try:
+            return [
+                TableParam.from_field(field) for field in fields]
+        except AttributeError:
+            return []
+
     def search(
             self, pos, size=1.0, format='all', intersect="overlaps",
             verbosity=2, **keywords):
@@ -300,6 +318,10 @@ class SIAService(DALService):
         return SIAQuery(
             self.baseurl, pos, size, format, intersect, verbosity, **keywords)
 
+    def describe(self):
+        print(self.description)
+        print()
+
 
 class SIAQuery(DALQuery):
     """
@@ -889,3 +911,12 @@ class SIARecord(SodaRecordMixin, DatalinkRecordMixin, Record):
         the format and choose an extension accordingly.
         """
         return mime2extension(self.format, default)
+
+    def broadcast_samp(self, client_name=None):
+        """
+        Broadcast the image to ``client_name`` via SAMP
+        """
+        with samp.connection() as conn:
+            samp.send_image_to(
+                conn, self.getdataurl(), client_name,
+                name=self.suggest_dataset_basename())
