@@ -24,12 +24,18 @@ endpoint.
 from __future__ import (
     absolute_import, division, print_function, unicode_literals)
 
+from pyvo.io.vosi.vodataservice import TableParam
+
 from astropy.units import Quantity, Unit
 from astropy.units import spectral as spectral_equivalencies
-from .query import (DALResults, DALQuery, DALService, Record)
+from astropy.io.votable.tree import Field
+from astropy.table import Table
+
+from .query import DALResults, DALQuery, DALService, Record
 
 __all__ = ["search", "SLAService", "SLAQuery", "SLAResults", "SLARecord"]
 
+
 def search(baseurl, wavelength, **keywords):
     """
     submit a simple SLA query that requests spectral lines within a
@@ -79,6 +85,45 @@ class SLAService(DALService):
         """
         super(SLAService, self).__init__(baseurl,)
 
+    def _get_metadata(self):
+        """
+        download the metadata resource
+        """
+        if not hasattr(self, "_metadata"):
+            query = self.create_query(request='getCapabilities')
+            metadata = query.execute_votable()
+
+            setattr(self, "_metadata", metadata)
+
+    @property
+    def description(self):
+        """
+        the service description.
+        """
+        self._get_metadata()
+
+        try:
+            return getattr(self, "_metadata", None).description
+        except AttributeError:
+            return None
+
+    @property
+    def columns(self):
+        """
+        the available columns on this service
+        """
+        self._get_metadata()
+        fields = filter(
+            lambda field_or_param: isinstance(field_or_param, Field),
+            self._metadata.iter_fields_and_params()
+        )
+
+        try:
+            return [
+                TableParam.from_field(field) for field in fields]
+        except AttributeError:
+            return []
+
     def search(self, wavelength, **keywords):
         """
         submit a simple SLA query to this service with the given constraints.
@@ -147,6 +192,34 @@ class SLAService(DALService):
         """
         return SLAQuery(self.baseurl, wavelength, request, **keywords)
 
+    def describe(self):
+        print(self.description)
+        print()
+
+        rows = [(
+            col.name,
+            col.description,
+            col.unit,
+            col.ucd,
+            col.utype,
+            col.datatype.arraysize,
+            col.datatype.content,
+        ) for col in self.columns]
+
+        names = (
+            'name',
+            'description',
+            'unit',
+            'ucd',
+            'utype',
+            'arraysize',
+            'datatype',
+        )
+
+        table = Table(rows=rows, names=names)
+        table.pprint(
+            max_lines=-1, max_width=-1, show_unit=False, show_dtype=False)
+
 
 class SLAQuery(DALQuery):
     """
@@ -188,42 +261,49 @@ class SLAQuery(DALQuery):
         """
         super(SLAQuery, self).__init__(baseurl)
 
-        if wavelength:
+        if wavelength is not None:
             self.wavelength = wavelength
 
         self.request = request
 
-        self.update({key.upper(): value for key, value in keywords.items()})
-
     @property
     def wavelength(self):
         """
-        the bandwidth range the observations belong to.
+        the frequency/wavelength range the observations belong to.
         """
         return getattr(self, "_wavelength", None)
-    @wavelength.setter
-    def wavelength(self, val):
-        setattr(self, "_wavelength", val)
 
-        if not isinstance(val, Quantity):
-            # assume meters
-            val = val * Unit("meter")
+    @wavelength.setter
+    def wavelength(self, wavelength):
+        setattr(self, "_wavelength", wavelength)
+
+        if not isinstance(wavelength, Quantity):
+            valerr = ValueError(
+                'Wavelength range must be a sequence with exactly two values',
+                'expressing a frequency or wavelength range')
+
             try:
-                if len(val) != 2:
-                    raise ValueError(
-                        "wavelength must be specified with exactly two values")
+                # assume meters
+                wavelength = wavelength * Unit("meter")
+            except ValueError:
+                raise valerr
+
+            try:
+                if len(wavelength) != 2:
+                    raise valerr
             except TypeError:
-                raise ValueError(
-                    "wavelength must be specified with exactly two values")
+                raise valerr
+
         # transform to meters
-        val = val.to(
+        wavelength = wavelength.to(
             Unit("m"), equivalencies=spectral_equivalencies())
-        # frequency is counter-proportional to wavelength, so we just sort it to
-        # have the right order again
-        val.sort()
+        # frequency is counter-proportional to wavelength, so we just sort it
+        # to have the right order again
+        wavelength.sort()
 
         self["WAVELENGTH"] = "{start}/{end}".format(
-            start=val.value[0], end=val.value[1])
+            start=wavelength.value[0], end=wavelength.value[1])
+
     @wavelength.deleter
     def wavelength(self):
         delattr(self, "_wavelength")
@@ -235,10 +315,12 @@ class SLAQuery(DALQuery):
         the type of service operation which is being performed
         """
         return getattr(self, "_request", None)
+
     @request.setter
     def request(self, val):
         setattr(self, "_request", val)
         self["REQUEST"] = val
+
     @request.deleter
     def request(self):
         delattr(self, "_request")
@@ -313,8 +395,8 @@ class SLAResults(DALResults):
         """
         return a representation of a sla result record that follows
         dictionary semantics. The keys of the dictionary are those returned by
-        this instance's fieldnames attribute. The returned record has additional
-        image-specific properties
+        this instance's fieldnames attribute. The returned record has
+        additional image-specific properties
 
         Parameters
         ----------
