@@ -12,17 +12,21 @@ from distutils.version import LooseVersion
 
 import requests
 
+from astropy.extern import six
 from astropy.io.votable import parse as votableparse
 
 from .query import (
-    DALResults, DALQuery, DALService, Record, UploadList, DALServiceError,
-    DALQueryError)
+    DALResults, DALQuery, DALService, Record, UploadList,
+    DALServiceError, DALQueryError)
 from .vosi import AvailabilityMixin, CapabilityMixin, VOSITables
 from .adhoc import DatalinkResultsMixin, DatalinkRecordMixin, SodaRecordMixin
 
 from ..io import vosi, uws
 from ..io.vosi import tapregext as tr
 
+from ..utils.formatting import para_format_desc
+from ..utils.http import session as s
+
 __all__ = [
     "search", "escape", "TAPService", "TAPQuery", "AsyncTAPJob", "TAPResults"]
 
@@ -96,7 +100,7 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         if self._tables is None:
             tables_url = '{0}/tables'.format(self.baseurl)
 
-            response = requests.get(tables_url, stream=True)
+            response = s.get(tables_url, stream=True)
 
             try:
                 response.raise_for_status()
@@ -149,7 +153,8 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
     @property
     def upload_methods(self):
         """
-        a list of upload methods in form of IVOA identifiers
+        a list of upload methods in form of
+        :py:class:`~pyvo.io.vosi.tapregext.UploadMethod` objects
         """
         upload_methods = []
         for capa in self.capabilities:
@@ -296,6 +301,33 @@ class TAPService(DALService, AvailabilityMixin, CapabilityMixin):
         return TAPQuery(
             self.baseurl, query, mode, language, maxrec, uploads, **keywords)
 
+    def describe(self, width=None):
+        """
+        Print a summary description of this service.
+
+        This includes the interface capabilities, and the content description
+        if it doesn't contains multiple data collections (in other words, it is
+        not a TAP service).
+        """
+        if len(self.tables) == 1:
+            description = next(self.tables.values()).description
+
+            if width:
+                description = para_format_desc(description, width)
+
+            print(description)
+            print()
+
+        capabilities = filter(
+            lambda x: not six.text_type(x.standardid).startswith(
+                'ivo://ivoa.net/std/VOSI'),
+            self.capabilities
+        )
+
+        for cap in capabilities:
+            cap.describe()
+            print()
+
 
 class AsyncTAPJob(object):
     """
@@ -359,19 +391,21 @@ class AsyncTAPJob(object):
         except Exception:
             pass
 
-    def _update(self, wait_for_statechange=False):
+    def _update(self, wait_for_statechange=False, timeout=10.):
         """
         updates local job infos with remote values
         """
         try:
             if wait_for_statechange:
-                response = requests.get(self.url, stream=True, params={
-                    "WAIT": "-1"
-                })
+                response = s.get(
+                    self.url, stream=True, timeout=timeout, params={
+                        "WAIT": "-1"
+                    }
+                )
             else:
-                response = requests.get(self.url, stream=True)
+                response = s.get(self.url, stream=True, timeout=timeout)
             response.raise_for_status()
-        except requests.exceptions.RequestException as ex:
+        except requests.RequestException as ex:
             raise DALServiceError.from_except(ex, self.url)
 
         # requests doesn't decode the content by default
@@ -421,11 +455,11 @@ class AsyncTAPJob(object):
     @execution_duration.setter
     def execution_duration(self, value):
         try:
-            response = requests.post(
+            response = s.post(
                 "{}/executionduration".format(self.url),
                 data={"EXECUTIONDURATION": str(value)})
             response.raise_for_status()
-        except requests.exceptions.RequestException as ex:
+        except requests.RequestException as ex:
             raise DALServiceError.from_except(ex, self.url)
 
         self._update()
@@ -457,11 +491,11 @@ class AsyncTAPJob(object):
             pass
 
         try:
-            response = requests.post(
+            response = s.post(
                 "{}/destruction".format(self.url),
                 data={"DESTRUCTION": value.strftime("%Y-%m-%dT%H:%M:%SZ")})
             response.raise_for_status()
-        except requests.exceptions.RequestException as ex:
+        except requests.RequestException as ex:
             raise DALServiceError.from_except(ex, self.url)
 
         self._update()
@@ -480,7 +514,53 @@ class AsyncTAPJob(object):
         job owner (if applicable)
         """
         self._update()
-        return self._job.owner
+        return self._job.ownerid
+
+    @property
+    def query(self):
+        """
+        the job query
+        """
+        self._update()
+        for parameter in self._job.parameters:
+            if parameter.id_ == 'query':
+                return parameter.content
+        return ''
+
+    @query.setter
+    def query(self, query):
+        try:
+            response = s.post(
+                '{}/parameters'.format(self.url),
+                data={"QUERY": query})
+            response.raise_for_status()
+        except requests.RequestException as ex:
+            raise DALServiceError.from_except(ex, self.url)
+
+        self._update()
+
+    def upload(self, **kwargs):
+        """
+        upload a table to the job. the job must not been started.
+        """
+        uploads = UploadList.fromdict(kwargs)
+        files = {
+            upload.name: upload.fileobj()
+            for upload in uploads
+            if upload.is_inline
+        }
+
+        try:
+            response = s.post(
+                '{}/parameters'.format(self.url),
+                data={'UPLOAD': uploads.param()},
+                files=files
+            )
+            response.raise_for_status()
+        except requests.RequestException as ex:
+            raise DALServiceError.from_except(ex, self.url)
+
+        self._update()
 
     @property
     def results(self):
@@ -526,10 +606,10 @@ class AsyncTAPJob(object):
         starts the job / change phase to RUN
         """
         try:
-            response = requests.post(
+            response = s.post(
                 '{}/phase'.format(self.url), data={"PHASE": "RUN"})
             response.raise_for_status()
-        except requests.exceptions.RequestException as ex:
+        except requests.RequestException as ex:
             raise DALServiceError.from_except(ex, self.url)
 
         return self
@@ -539,16 +619,15 @@ class AsyncTAPJob(object):
         aborts the job / change phase to ABORT
         """
         try:
-            response = requests.post(
+            response = s.post(
                 '{}/phase'.format(self.url), data={"PHASE": "ABORT"})
             response.raise_for_status()
-        except requests.exceptions.RequestException as ex:
+        except requests.RequestException as ex:
             raise DALServiceError.from_except(ex, self.url)
 
         return self
 
-    def wait(
-            self, phases=None):
+    def wait(self, phases=None, timeout=600.):
         """
         waits for the job to reach the given phases.
 
@@ -572,7 +651,7 @@ class AsyncTAPJob(object):
             "QUEUED", "EXECUTING", "RUN", "COMPLETED", "ERROR", "UNKNOWN"}
 
         while True:
-            self._update(wait_for_statechange=True)
+            self._update(wait_for_statechange=True, timeout=timeout)
             # use the cached value
             cur_phase = self._job.phase
 
@@ -595,9 +674,9 @@ class AsyncTAPJob(object):
         deletes the job. this object will become invalid.
         """
         try:
-            response = requests.post(self.url, data={"ACTION": "DELETE"})
+            response = s.post(self.url, data={"ACTION": "DELETE"})
             response.raise_for_status()
-        except requests.exceptions.RequestException as ex:
+        except requests.RequestException as ex:
             raise DALServiceError.from_except(ex, self.url)
 
         self._url = None
@@ -619,7 +698,7 @@ class AsyncTAPJob(object):
         returns the result votable if query is finished
         """
         try:
-            response = requests.get(self.result_uri, stream=True)
+            response = s.get(self.result_uri, stream=True)
             response.raise_for_status()
         except requests.RequestException as ex:
             self._update()
@@ -745,7 +824,7 @@ class TAPQuery(DALQuery):
             if upload.is_inline
         }
 
-        response = requests.post(
+        response = s.post(
             url, data=self, stream=True, files=files)
         # requests doesn't decode the content by default
         response.raw.read = partial(response.raw.read, decode_content=True)
