@@ -6,7 +6,12 @@ from __future__ import (
     absolute_import, division, print_function, unicode_literals)
 
 from functools import partial
-from contextlib import ExitStack
+
+try:
+    from contextlib import ExitStack
+except ImportError:
+    from contextlib2 import ExitStack
+
 import re
 from io import BytesIO
 
@@ -30,6 +35,9 @@ job_re_path = re.compile('^/tap/async/([0-9]+)')
 
 job_re_phase_full = re.compile('^http://example.com/tap/async/([0-9]+)/phase')
 
+job_re_parameters_full = re.compile(
+    '^http://example.com/tap/async/([0-9]+)/parameters')
+
 job_re_result_full = re.compile(
     '^http://example.com/tap/async/([0-9]+)/results/result')
 
@@ -39,7 +47,7 @@ def _test_image_results(results):
 
 
 @pytest.fixture()
-def sync(mocker):
+def sync_fixture(mocker):
     def callback(request, context):
         return get_pkg_data_contents('data/tap/obscore-image.xml')
 
@@ -50,7 +58,7 @@ def sync(mocker):
 
 
 @pytest.fixture()
-def async(mocker):
+def async_fixture(mocker):
     class Callback(object):
         def __init__(self):
             self._jobs = dict()
@@ -81,8 +89,7 @@ def async(mocker):
             self._jobs[newid] = job
 
         def job(self, request, context):
-            jobid = int(job_re_path.match(request.path)[1])
-            data = dict(parse_qsl(request.body))
+            jobid = int(job_re_path.match(request.path).group(1))
 
             if request.method == 'GET':
                 job = self._jobs[jobid]
@@ -90,13 +97,14 @@ def async(mocker):
                 job.to_xml(io)
                 return io.getvalue()
             elif request.method == 'POST':
+                data = dict(parse_qsl(request.body))
                 action = data.get('ACTION')
 
                 if action == 'DELETE':
                     del self._jobs[jobid]
 
         def phase(self, request, context):
-            jobid = int(job_re_path.match(request.path)[1])
+            jobid = int(job_re_path.match(request.path).group(1))
 
             if request.method == 'GET':
                 phase = self._jobs[jobid].phase
@@ -119,11 +127,45 @@ def async(mocker):
 
                     try:
                         job.results[0] = result
-                    except IndexError:
+                    except (IndexError, TypeError):
                         job.results.append(result)
 
                 job.phase = newphase
 
+        def parameters(self, request, context):
+            jobid = int(job_re_path.match(request.path).group(1))
+            job = self._jobs[jobid]
+
+            if request.method == 'GET':
+                pass
+            elif request.method == 'POST':
+                data = dict(parse_qsl(request.body))
+
+                if 'QUERY' in data:
+                    assert data['QUERY'] == 'SELECT TOP 42 * FROM ivoa.obsCore'
+                    for param in job.parameters:
+                        if param.id_ == 'query':
+                            param.content = data['QUERY']
+                if 'UPLOAD' in data:
+                    for param in job.parameters:
+                        if param.id_ == 'upload':
+                            uploads1 = {data[0]: data[1] for data in [
+                                data.split(',') for data
+                                in data['UPLOAD'].split(';')
+                            ]}
+
+                            uploads2 = {data[0]: data[1] for data in [
+                                data.split(',') for data
+                                in param.content.split(';')
+                            ]}
+
+                            uploads1.update(uploads2)
+
+                            param.content = ';'.join([
+                                '{}={}'.format(key, value) for key, value
+                                in uploads1.items()
+                            ])
+
         def result(self, request, context):
             # jobid = int(job_re_path.match(request.path)[1])
             return get_pkg_data_contents('data/tap/obscore-image.xml')
@@ -142,6 +184,11 @@ def async(mocker):
             'phase': stack.enter_context(mocker.register_uri(
                 requests_mock.ANY, job_re_phase_full, content=callback.phase
             )),
+            'parameters': stack.enter_context(mocker.register_uri(
+                requests_mock.ANY, job_re_parameters_full,
+                content=callback.parameters
+            )),
+
             'result': stack.enter_context(mocker.register_uri(
                 'GET', job_re_result_full, content=callback.result
             ))
@@ -197,7 +244,10 @@ def test_escape():
             "SELECT * FROM ivoa.obscore WHERE dataproduct_type = ''image''")
 
 
-@pytest.mark.usefixtures('sync')
+@pytest.mark.usefixtures('sync_fixture')
+@pytest.mark.filterwarnings("ignore::astropy.io.votable.exceptions.W27")
+@pytest.mark.filterwarnings("ignore::astropy.io.votable.exceptions.W48")
+@pytest.mark.filterwarnings("ignore::astropy.io.votable.exceptions.W06")
 def test_search():
     results = search('http://example.com/tap', "SELECT * FROM ivoa.obscore")
 
@@ -224,7 +274,7 @@ class TestTAPService(object):
 
         assert list(tables.keys()) == ['test.table1', 'test.table2']
 
-        table1, table2 = list(tables.iter_tables())
+        table1, table2 = list(tables)
         self._test_tables(table1, table2)
 
     @pytest.mark.usefixtures('capabilities')
@@ -253,25 +303,34 @@ class TestTAPService(object):
         assert upload_methods[3].ivo_id == (
             'ivo://ivoa.net/std/TAPRegExt#upload-http')
 
-    @pytest.mark.usefixtures('sync')
+    @pytest.mark.usefixtures('sync_fixture')
+    @pytest.mark.filterwarnings("ignore::astropy.io.votable.exceptions.W27")
+    @pytest.mark.filterwarnings("ignore::astropy.io.votable.exceptions.W48")
+    @pytest.mark.filterwarnings("ignore::astropy.io.votable.exceptions.W06")
     def test_run_sync(self):
         service = TAPService('http://example.com/tap')
         results = service.run_sync("SELECT * FROM ivoa.obscore")
         _test_image_results(results)
 
-    @pytest.mark.usefixtures('sync')
+    @pytest.mark.usefixtures('sync_fixture')
+    @pytest.mark.filterwarnings("ignore::astropy.io.votable.exceptions.W27")
+    @pytest.mark.filterwarnings("ignore::astropy.io.votable.exceptions.W48")
+    @pytest.mark.filterwarnings("ignore::astropy.io.votable.exceptions.W06")
     def test_search(self):
         service = TAPService('http://example.com/tap')
         results = service.search("SELECT * FROM ivoa.obscore")
         _test_image_results(results)
 
-    @pytest.mark.usefixtures('async')
+    @pytest.mark.usefixtures('async_fixture')
+    @pytest.mark.filterwarnings("ignore::astropy.io.votable.exceptions.W27")
+    @pytest.mark.filterwarnings("ignore::astropy.io.votable.exceptions.W48")
+    @pytest.mark.filterwarnings("ignore::astropy.io.votable.exceptions.W06")
     def test_run_async(self):
         service = TAPService('http://example.com/tap')
         results = service.run_async("SELECT * FROM ivoa.obscore")
         _test_image_results(results)
 
-    @pytest.mark.usefixtures('async')
+    @pytest.mark.usefixtures('async_fixture')
     def test_submit_job(self):
         service = TAPService('http://example.com/tap')
         job = service.submit_job(
@@ -286,3 +345,23 @@ class TestTAPService(object):
         job.run()
         job.wait()
         job.delete()
+
+    @pytest.mark.usefixtures('async_fixture')
+    def test_modify_job(self):
+        service = TAPService('http://example.com/tap')
+        job = service.submit_job(
+            "SELECT * FROM ivoa.obscore", uploads={
+                'one': 'http://example.com/uploads/one'
+            })
+        job.query = "SELECT TOP 42 * FROM ivoa.obsCore"
+        job.upload(two='http://example.com/uploads/two')
+
+        for parameter in job._job.parameters:
+            if parameter.id_ == 'query':
+                assert parameter.content == 'SELECT TOP 42 * FROM ivoa.obsCore'
+                break
+            elif parameter.id_ == 'upload':
+                assert (
+                    'one=http://example.com/uploads/one' in parameter.content)
+                assert (
+                    'two=http://example.com/uploads/two' in parameter.content)
